import argparse
import sys
import os

from .random_word import RandomWord, NoWordsToChoseFrom
from .random_sentence import RandomSentence
from .cmdline import WonderwordsCommandLine


def main():
    parser = argparse.ArgumentParser(
        prog="wonderwords",
        description="""Generate random words and sentences from the command line.
        Here is a full list of available commands. To learn more about each
        command, go to the documentation at https://wonderwords.readthedocs.io
        """,
        epilog="""Thanks to all contributors who made this possible.
        To contribute, go to https://github.com/mrmaxguns/wonderwordsmodule
        """,
    )

    #
    # Base commands
    #
    parser.add_argument(
        "-w",
        "--word",
        "--random-word",
        action="store_true",
        help="generate a random word",
    )

    parser.add_argument(
        "-f",
        "--filter",
        action="store_true",
        help="filter a list of words matching the criteria specified",
    )

    parser.add_argument(
        "-l",
        "--list",
        action="store",
        type=int,
        help="return a list of words of a certain length",
    )

    parser.add_argument(
        "-s",
        "--sentence",
        action="store",
        type=str,
        choices=[
            "bb",
            "ss",
            "bba",
            "s",
        ],
        help="return a sentence based on the structure chosen",
    )

    parser.add_argument(
        "-v", "--version", action="store_true", help="Print the version number and exit"
    )

    #
    # Settings and modifiers
    #
    parser.add_argument(
        "-sw",
        "--starts-with",
        action="store",
        default="",
        type=str,
        help="specify what string the random word(s) should start with",
    )

    parser.add_argument(
        "-ew",
        "--ends-with",
        action="store",
        default="",
        type=str,
        help="specify what string the random word(s) should end with",
    )

    parser.add_argument(
        "-p",
        "--parts-of-speech",
        action="store",
        type=str,
        nargs="+",
        choices=["nouns", "verbs", "adjectives"],
        help="specify to only include certain parts of speech (by default all parts of speech are included)",
    )

    parser.add_argument(
        "-min",
        "--word-min-length",
        action="store",
        type=int,
        help="specify the minimum length of the word(s)",
    )

    parser.add_argument(
        "-max",
        "--word-max-length",
        action="store",
        type=int,
        help="specify the maximum length of the word(s)",
    )

    parser.add_argument(
        "-r",
        "--regex",
        "--re",
        "--regular-expression",
        action="store",
        type=str,
        help="specify a python-style regular expression that every word must match",
    )

    parser.add_argument(
        "-d",
        "--delimiter",
        default=", ",
        type=str,
        help="Specify the delimiter to put between a list of words, default is ', '",
    )

    args = parser.parse_args()
    mode = get_mode(args)
    handle_mode(mode, args)


def get_mode(arguments):
    if arguments.version:
        MODE = "version"
    elif arguments.word:
        MODE = "word"
    elif arguments.filter:
        MODE = "filter"
    elif arguments.list is not None:
        MODE = "list"
    elif arguments.sentence is not None:
        MODE = "sentence"
    else:
        MODE = None

    return MODE


def handle_mode(mode, arguments):
    command_line = WonderwordsCommandLine()

    if mode == "version":
        command_line.version()
        quit()

    if mode == "word" or mode == "filter" or mode == "list":
        word_parser = RandomWord()

    if mode == "sentence":
        sent_parser = RandomSentence()

    if mode == "word":
        try:
            word = word_parser.word(
                starts_with=arguments.starts_with,
                ends_with=arguments.ends_with,
                include_parts_of_speech=arguments.parts_of_speech,
                word_min_length=arguments.word_min_length,
                word_max_length=arguments.word_max_length,
                regex=arguments.regex,
            )
        except NoWordsToChoseFrom:
            command_line.no_word()
            quit()

        command_line.word(word)
    elif mode == "filter":
        words = word_parser.filter(
            starts_with=arguments.starts_with,
            ends_with=arguments.ends_with,
            include_parts_of_speech=arguments.parts_of_speech,
            word_min_length=arguments.word_min_length,
            word_max_length=arguments.word_max_length,
            regex=arguments.regex,
        )

        command_line.words(words, delimiter=arguments.delimiter)

    elif mode == "list":
        try:
            words = word_parser.random_words(
                amount=arguments.list,
                starts_with=arguments.starts_with,
                ends_with=arguments.ends_with,
                include_parts_of_speech=arguments.parts_of_speech,
                word_min_length=arguments.word_min_length,
                word_max_length=arguments.word_max_length,
                regex=arguments.regex,
            )
        except NoWordsToChoseFrom:
            command_line.no_words()
            words = word_parser.random_words(
                amount=arguments.list,
                starts_with=arguments.starts_with,
                ends_with=arguments.ends_with,
                include_parts_of_speech=arguments.parts_of_speech,
                word_min_length=arguments.word_min_length,
                word_max_length=arguments.word_max_length,
                return_less_if_necessary=True,
                regex=arguments.regex,
            )

        command_line.words(words, delimiter=arguments.delimiter)
    elif mode == "sentence":
        if arguments.sentence == "bb":
            command_line.sentence(sent_parser.bare_bone_sentence())
        elif arguments.sentence == "ss":
            command_line.sentence(sent_parser.simple_sentence())
        elif arguments.sentence == "bba":
            command_line.sentence(sent_parser.bare_bone_with_adjective())
        else:
            command_line.sentence(sent_parser.sentence())
    else:
        command_line.intro()


if __name__ == "__main__":
    main()



from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.emoji import Emoji
from rich.padding import Padding
from rich.markdown import Markdown

from . import __version__


console = Console()

AVAILABLE_COMMANDS = """## Available Commands

* `wonderwords -w` - generate a random word
* `wonderwords -f` - get all words matching a certain criteria
* `wonderwords -l AMOUNT` - get a list of `AMOUNT` random words
* `wonderwords -s SENT_TYPE` - generate a random sentence of a certain type

For a list of all options, type `wonderwords -h`. To see a detailed and
comprehensive explanation of the commands, visit
[the documentation](https://wonderwords.readthedocs.io)
"""


class WonderwordsCommandLine:
    def print_title(self):
        title = Panel(
            Text(f"WONDERWORDS {__version__}", justify="center"),
            padding=1,
            style="bold navy_blue on white",
        )
        console.print(title)

    def print_commands(self):
        commands = Markdown(AVAILABLE_COMMANDS)
        console.print(commands)

    def version(self):
        console.print(
            f"Running wonderwords version {__version__}", style="navy_blue on white"
        )

    def intro(self):
        self.print_title()
        info_text = Text(
            f"No commands given {Emoji('disappointed_face')}",
            justify="center",
            style="bold",
        )
        console.print(Padding(info_text, pad=1))
        self.print_commands()

    def word(self, word):
        word_text = Text(word, style="bold white on navy_blue")
        console.print(word_text)

    def words(self, words, delimiter):
        word_text = Text(delimiter.join(words), style="bold white on navy_blue")
        console.print(word_text)

    def sentence(self, sent):
        sent_text = Text(sent, style="bold white on dark_green")
        console.print(sent_text)

    def no_word(self):
        console.print(
            "A word with the parameters specified does not exist! :anguished: ",
            style="white on red",
        )

    def no_words(self):
        console.print(
            "There weren't enough words that matched your request. All words available are listed below :anguished: ",
            style="white on red",
        )



"""
Generate structured sentences in which every word is random.
"""

import random
from typing import Optional, List

from .random_word import RandomWord

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources


VOWELS = ["a", "e", "i", "o", "u"]


def _present_tense(verb):
    """Convert a verb in the form of "to ____" to the present tense 3rd person
    form"""
    verb = verb.strip()
    lowercase_verb = verb.lower()
    if lowercase_verb.endswith(("ss", "ch", "x", "tch", "sh", "zz")):
        verb = verb + "es"
    elif lowercase_verb.endswith("y") and not verb.endswith(
        tuple([vowel + "y" for vowel in VOWELS])
    ):
        verb = verb[:-1] + "ies"
    else:
        verb = verb + "s"
    return verb


# The main class containing all the data and functions
class RandomSentence:
    """The RandomSentence provides an easy interface to generate structured
    sentences where each word is randomly generated.

    Example::

        >>> s = RandomSentence(nouns=["car", "cat", "mouse"], verbs=["eat"])
        >>> s2 = RandomSentence()

    :param nouns: a list of nouns that will be used to generate random nouns.
        Defaults to None.
    :type nouns: list, optional
    :param verbs: a list of verbs that will be used to generate random verbs.
        Defaults to None.
    :type verbs: list, optional
    :param adjectives: a list of adjectives that will be used to generate random
        adjectives. Defaults to None.
    :type adjectives: list, optional
    """

    def __init__(
        self,
        nouns: Optional[List[str]] = None,
        verbs: Optional[List[str]] = None,
        adjectives: Optional[List[str]] = None,
    ):
        self.noun = nouns or RandomWord.read_words("nounlist.txt")
        self.verb = verbs or RandomWord.read_words("verblist.txt")
        self.adjective = adjectives or RandomWord.read_words("adjectivelist.txt")

    # Randomly generate bare bone sentences
    def bare_bone_sentence(self):
        """Generate a bare-bone sentence in the form of
        ``The [subject (noun)] [predicate (verb)].``. For example:
        ``The cat runs.``.

        Example::

            >>> s.bare_bone_sentence()

        :return: string in the form of a bare bone sentence where each word is
            randomly generated
        :rtype: str
        """
        the_noun = random.choice(self.noun)
        the_verb = _present_tense(random.choice(self.verb))

        return f"The {the_noun} {the_verb}."

    def simple_sentence(self):
        """Generate a simple sentence in the form of
        ``The [subject (noun)] [predicate (verb)] [direct object (noun)].``. For
        example: ``The cake plays golf``.

        Example::

            >>> s.simple_sentence()

        :return: a string in the form of a simple sentence where each word is
            randomly generated
        :rtype: str
        """
        the_direct_object = random.choice(self.noun)
        the_bare_bone_sentence = self.bare_bone_sentence()[:-1]

        return f"{the_bare_bone_sentence} {the_direct_object}."

    def bare_bone_with_adjective(self):
        """Generate a bare-bone sentence with an adjective in the form of:
        ``The [(adjective)] [subject (noun)] [predicate (verb)].``. For example:
        ``The skinny cat reads.``

        Example::

            >>> s.bare_bone_with_adjective()

        :return: a string in the form of a bare-bone sentence with an adjective
            where each word is randomly generated
        :rtype: str
        """
        the_noun = random.choice(self.noun)
        the_verb = _present_tense(random.choice(self.verb))
        the_adjective = random.choice(self.adjective)

        return f"The {the_adjective} {the_noun} {the_verb}."

    def sentence(self):
        """Generate a simple sentence with an adjective in the form of:
        ``The [(adjective)] [subject (noun)] [predicate (verb)] [direct object (noun)].``.
        For example: ``The green orange likes food.``

        Example::

            >>> s.sentence()

        :return: a string in the form of a simple sentence with an adjective
            where each word is randomly generated
        :rtype: str
        """
        the_bare_bone_with_adjective = self.bare_bone_with_adjective()[:-1]
        the_direct_object = random.choice(self.noun)

        return f"{the_bare_bone_with_adjective} {the_direct_object}."
